# Copyright 2014, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#    Josef Skladanka <jskladan@redhat.com>

from oraculum import app, cache
from oraculum import data_providers
from oraculum.data_providers import PROVIDERS
from oraculum.utils import distgit, bz, fedocal, schedule, blockerbugs, meetbot, ACTION_TIMES_SPREAD

from flask import request, url_for, jsonify, Blueprint
from werkzeug.contrib.cache import SimpleCache

import urllib
import itertools
import importlib

# FIXME: Move this into a proper config file
CACHE_TIMEOUT = 50


@app.route('/api/v0/user/<fas_login>')
@cache.cached(timeout=CACHE_TIMEOUT)
def api_v0_user(fas_login):
    distgit_info = distgit.get_user(fas_login)
    bz_bugs = bz.get_user_bugs(fas_login)
    bz_bugs = [{'summary': b.summary, 'status': b.status, 'id': b.id} for b in bz_bugs]
    packages = [r['name'] for r in distgit_info.get('repos', [])] # FIXME add forksendpoints
    resp = {
        'user': fas_login,
        'packages': packages,
        'bugs': bz_bugs,
    }

    return jsonify(resp)

##
@app.route('/api/v0/meetings')
@cache.cached(timeout=CACHE_TIMEOUT)
def api_v0_meetings():
    mtgs = fedocal.get_qa_meetings()
    resp = {
        'meetings': mtgs,
    }
    return jsonify(resp)
##
@app.route('/api/v0/last_qa_meeting')
@cache.cached(timeout=CACHE_TIMEOUT)
def api_v0_last_qa_meeting():
    resp = meetbot.get_last_qa_meeting()

    return jsonify(resp)


###
@app.route('/api/v0/schedule')
@cache.cached(timeout=CACHE_TIMEOUT)
def api_v0_schedule_bikeshed():
    sched = schedule.get_schedule()
    resp = {
        'schedule': sched,
    }
    return jsonify(resp)

##
@app.route('/api/v0/blockerbugs')
@cache.cached(timeout=CACHE_TIMEOUT)
def api_v0_blockerbugs():
    blockers = blockerbugs.get_blockerbugs()
    release = schedule.current_devel()
    resp = {
        'release': release,
        'blockerbugs': blockers,
    }
    return jsonify(resp)

@app.route('/api/v0/release')
@cache.cached(timeout=CACHE_TIMEOUT)
def api_v0_release():
    stable = schedule.current_stable()
    devel = schedule.current_devel()
    resp = {
        'stable': stable,
        'devel': devel,
    }
    return jsonify(resp)

