# Copyright 2014, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#    Josef Skladanka <jskladan@redhat.com>

from oraculum import app, cache
from oraculum.data_providers import PROVIDERS
from oraculum.utils import fedocal, schedule, blockerbugs, meetbot

from flask import request, url_for, jsonify
from werkzeug.contrib.cache import SimpleCache

import urllib
import itertools


# FIXME: Move this into a proper config file
CACHE_TIMEOUT = 3600

@app.route('/api/v1/landing_page')
@cache.cached(timeout=CACHE_TIMEOUT)
def api_v0_meetings():
    mtgs = fedocal.get_qa_meetings()
    last_qa_meeting = meetbot.get_last_qa_meeting()
    sched = schedule.get_schedule()
    blockers = blockerbugs.get_blockerbugs()
    stable = schedule.current_stable()
    devel = schedule.current_devel()

    resp = {
        'meetings': mtgs,
        'last_qa_meeting': last_qa_meeting,
        'schedule': sched,
        'blockerbugs': blockers,
        'stable': stable,
        'devel': devel,
      }
    return jsonify(resp)

CACHE = SimpleCache()
def get_actions(provider=None, tags=None):
    actions = CACHE.get('api_v1_actions')
    TIMEOUT = 3600 # FIXME - make the caching configurable (ideally per-provider)
    if actions is None:
        actions = []
        for p_name, p_module in PROVIDERS.items():
            actions.extend(p_module.get_actions())
        CACHE.set('api_v1_actions', actions, timeout=TIMEOUT)

    if provider:
        actions = [a for a in actions if a['provider'] == provider]
    if tags:
        tags = set(tags.split(','))
        actions = [a for a in actions if tags.intersection(a['tags'])]
    return actions


@app.route('/api/v1/actions')
def api_v1_actons():
    provider = request.args.get('provider')
    tags = request.args.get('tags')

    resp = {'actions': get_actions(provider, tags)}
    return jsonify(resp)


@app.route('/api/v1/actions/providers')
def api_v1_actions_providers():
    resp = {'providers': [p.INFO for p in PROVIDERS.values()]}
    return jsonify(resp)


@app.route('/api/v1/actions/tags')
def api_v1_actions_tags():
    provider = request.args.get('provider')
    actions = get_actions(provider)

    resp = {'tags': []}
    if actions:
        resp['tags'] = list(set(itertools.chain(*[a['tags'] for a in actions])))

    return jsonify(resp)

@app.route('/api/v1/actions/all')
def api_v1_actions_all():

    resp = {
        'providers': [p.INFO for p in PROVIDERS.values()],
        'actions': {p: get_actions(p) for p in PROVIDERS.keys()}
    }

    return jsonify(resp)

@app.route('/')
@app.route('/index')
def index():
    endpoints = []

    for rule in app.url_map.iter_rules():

        if rule.endpoint in ['index', 'static']:
            continue

        options = {}
        for arg in rule.arguments:
            options[arg] = "[{0}]".format(arg)

        url = url_for(rule.endpoint, **options)
        endpoints.append(urllib.parse.unquote(url))

    resp = {
        'endpoints': endpoints
    }

    return jsonify(resp)
