import icalendar
import requests
import datetime
import pytz


QA_CAL_URL = "https://apps.fedoraproject.org/calendar/ical/QA/"
CAL_API_URL = "https://apps.fedoraproject.org/calendar/api/meetings"
EVENT_LINK = "https://apps.fedoraproject.org/calendar/QA/#m%s"


def _to_datetime(date, tz=pytz.UTC):
    time = datetime.datetime.min.time()
    return datetime.datetime.combine(date, time).astimezone(tz)


"""
def get_qa_meetings():
    today = datetime.datetime.utcnow().astimezone(datetime.timezone.utc)
    resp = requests.get(QA_CAL_URL)
    data = []
    if resp.status_code == 200:
        ical_raw = resp.text
        ical = icalendar.Calendar.from_ical(ical_raw)
        for c in ical.walk():
            if c.name == "VEVENT":
                start = c.get('dtstart').dt
                end = c.get('dtend').dt

                if not isinstance(start, datetime.datetime):
                    start = _to_datetime(start)
                    end = _to_datetime(end)

                start_text = start.astimezone(datetime.timezone.utc).strftime("%d %b %Y")
                end_text = end.astimezone(datetime.timezone.utc).strftime("%d %b %Y")

                data.append({
                    'summary': c.get('summary'),
                    'start': start_text,
                    'end': end_text,
                    'dtdate': end
                })
    data = [d for d in data if d['dtdate'] >= today]
    return data
"""


def get_qa_meetings():
    today = datetime.date.today()
    end_date = today + datetime.timedelta(days=7)

    resp = requests.get(CAL_API_URL + "?start=%s&end=%s" % (today, end_date))
    data = []
    if resp.status_code == 200:
        cal_raw = resp.json()

        for meeting in cal_raw['meetings']:
            if meeting['calendar_name'] != 'QA':
                continue

            date_start = datetime.datetime.strptime(meeting['meeting_date'],"%Y-%m-%d").strftime("%d %b %Y")
            date_end = datetime.datetime.strptime(meeting['meeting_date_end'],"%Y-%m-%d").strftime("%d %b %Y")

            #FIXME properly remove seconds
            time_start = meeting['meeting_time_start'][:-3]
            time_end = meeting['meeting_time_stop'][:-3]


            data.append({
                'summary': meeting['meeting_name'],
                'start': date_start + ' ' + time_start,
                'end': time_end,
                'link': EVENT_LINK % meeting['meeting_id'],
                'info': meeting['meeting_information'],
                'fullday': meeting['meeting_time_start'] == meeting['meeting_time_stop'],
            })

    return data
