import icalendar
import requests
import datetime
import pytz

SCHEDULE_URL = "https://fedorapeople.org/groups/schedule/f-%d/f-%d-key.ics"
RELEASES_URL = "https://getfedora.org/releases.json"


def current_stable():
    resp = requests.get(RELEASES_URL)
    if resp.status_code == 200:
        releases = resp.json()
        return max([int(r['version']) for r in releases if 'Beta' not in r['version']])  # FIXME
    else:
        return -1  # FIXME


def current_devel():
    return current_stable() + 1


def _is_relevant(summary):
    relevant = ['Branch Fedora', 'Beta Freeze starts', 'Beta Release', 'Final Freeze starts', 'Final Release']
    return any([word in summary for word in relevant])


def get_schedule():
    today = datetime.datetime.utcnow()
    today = today.replace(tzinfo=pytz.UTC)
    resp = requests.get(SCHEDULE_URL % (current_devel(), current_devel()))  # FIXME
    data = []
    if resp.status_code == 200:  # FIXME
        ical_raw = icalendar.Calendar.from_ical(resp.text)
        for c in ical_raw.walk():
            if c.name == "VEVENT":
                summary = c.get('summary')
                date = c.get('dtstart').dt.astimezone(pytz.UTC)
                if _is_relevant(summary):
                    print(c)
                    data.append({
                        'summary': summary,
                        'date': date.strftime("%d %b %Y"),
                        'dtdate': date,
                        'current': date <= today  # mark all until now as current and clean up later
                    })

    data = sorted(data, key=lambda c: c['dtdate'])

    # leave the most recent 'current' milestone and clean up rest
    found = False
    for d in data[::-1]:
        if d['current'] and not found:
            found = True
            continue
        d['current'] = False

    # FIXME handle this better
    prebranch = {
        'summary': 'Rawhide',
        'date': today.strftime("%d %b %Y"),
        'dtdate': today,
        'current': not any([d['current'] for d in data]),
    }
    data = [prebranch] + data

    return data
